// myfocuser example - DRV8825 FULL
// ONLY FOR USE DRV8825 AND EASYDRIVER Driver Board - supports 1.5A at 12VDC and microstepping
// BIPOLAR STEPPER MOTORS ONLY
// SUPPORTS LCD
// SUPPORTS TEMPERATURE PROBE
// SUPPORTS KEYES 040 ROTARY ENCODER (other encoders will probably not work with this code)
//
// (c) Copyright Robert Brown 2014-1015. All Rights Reserved.
// The schematic, code and ideas for myFocuser are released into the public domain. Users are free to implement these but
// may NOT sell this project or projects based on this project for commercial gain without express written permission
// granted from the author. Schematics, Code, Firmware, Ideas, Applications, Layout are protected by Copyright Law.
// Permission is NOT granted to any person to redistribute, market, manufacture or sell for commercial gain the myFocuser
// products, ideas, circuits, builds, variations and units described or discussed herein or on this site.
// Permission is granted for personal use only.

#include <Arduino.h>
#include <OneWire.h>                // needed for DS18B20 temperature probe
#include <DallasTemperature.h>      // needed for DS18B20 temperature probe
#include <EEPROM.h>                 // needed for EEPROM
#include "eepromanything.h"         // needed for EEPROM
#include "Quadrature.h"             // needed for Keyes 040 rotary encoder
#define EEPROMSIZE 1024               // ATMEGA328P 1024 EEPROM

// these are stored in EEPROM - all variables in a structure
struct config_t {
  int validdata;       // if this is 99 then data is valid
  long fposition;       // last focuser position
  long maxstep;         // max steps
  int stepmode;                  // stepping mode
  // indicates stepmode, full, half, 1/4, 1/8. 1/16. 1/32 [1.2.4.8.16.32]
  long tempwaitdelay;         // interval in milliseconds to wait between temperature requests, default is 5s, configurable
  boolean ReverseDirection;      // reverse direction
  boolean coilPwr;               // coil pwr
} myfocuser;

int datasize;      // will hold size of the struct myfocuser - 6 bytes
int nlocations;    // number of storage locations available in EEPROM
int currentaddr;   // will be address in eeprom of the data stored
boolean writenow;     // should we update values in eeprom
boolean found;        // did we find any stored values?
long previousMillis = 0L;   // used as a delay whenever the EEPROM settings need to be updated
long interval = 10000L;     // interval in milliseconds to wait after a move before writing settings to EEPROM, 10s
long temppreviousMillis = 0L;  // used as a delay whenever a temp request comes in (decide whether to send last value)
long numwrites = 0L;        // number of eeprom writes this session
float tempoffsetval = 0;    // used to calibrate temperature offset, moonlite focuser
int stepdelay = 2;          // used by moonlite focuser
int tempcomp = 0;           // used by moonlite focuser

const String programName = "myFocuser_DRV8F";
const String programVersion = "1.5.1re";

// define temperature probe
#define ch1temp 2      // temperature probe on pin 2, use 4.7k pullup
// define objects for temperature probes
OneWire oneWirech1(ch1temp);            // setup temperature probe 1
// Pass our oneWire reference to Dallas Temperature library
DallasTemperature sensor1(&oneWirech1); // probe

// Stepper Motor stuff, control pins for DRV8825 board
#define myDir     3  // Direction
#define myStep    4  // Step
#define myM0      5  // microstepping lines
#define myM1      6
#define myM2      7
// m0/m1/m2 sets stepping mode 000 = F, 100 = 1/2, 010 = 1/4, 110 = 1/8, 001 = 1/16, 101 = 1/32
// steps per revolution = 200, 400, 800, 1600, 6400
// must set the current limiting for microstepping to work correctly

// ENABLE - Setting CoilPwr controls the disable or enable state of the EASYDRIVER board outputs
int boardstate;
#define OUTPUTENABLED 1
#define OUTPUTDISABLED 0
#define myEnable  8        // For DRV8825 and Easydriver boards, wire /ENABLE pin to D8 input
// NOTE: If using microstepping, coil power should always be ON and do NOT disable the board

// stepontime - time in microseconds that coil power is ON for one step, board requires 2us pulse
int stepontime = 5;              // changed 20141230 - seems better when 1/4 and 1/8 stepping
// motorSpeed - time in milisecobds of delay between stepper pulses
int motorSpeedSlow = 20;         // for 1000 half steps = about 56s
int motorSpeedMed = 10;          // for 1000 half steps = about 36s
int motorSpeedFast = 3;          // for 1000 half steps = about 18s
int motorSpeed = motorSpeedMed;  // changed 20141230 - seems better when 1/4 and 1/8 stepping

// define Rotary Encoder Pins
Quadrature myrotaryencoder(10, 11);  //  Rotary encoder is connected to pins 9 and 10
#define encoderSWPin 9             // encoder push sw on pin 11
long encswpress;                    // how long PB of encoder is held down
int encswval;                       // used to determine increment value of encoder, +1, +10, +100 per tick
boolean RESWVal;                    // state of Rotary Encoder Switch

// NOTE: Use Analogue pins
// define IN and OUT LEDS, associated with PB and stepper moves
#define bledIN A5
#define gledOUT A4
#define led1 A1
#define led2 A2
#define led3 A3
// define Buzzer
#define Buzzer A0

// Default initial positions if not set/overriden by Ascom Driver or Winapp
long currentPosition = 5000L;   // current position
long targetPosition = 5000L;    // target position
long maxFocuserLimit = 65000L;  // arbitary focuser limit to be compatible with moonlite
long maxSteps = 10000L;         // maximum position of focuser
long maxIncrement = 1000L;      // maximum number of steps permitted in one move
long minimumPosition = 0L;      // minimum position to avoid focuser damage
boolean gotonewposition = false;  // used by moonlite after an SN command followed by a FG

char inChar;                  // used to read a character from serial port
boolean isMoving = false;     // is the motor currently moving
long pos;

#define MAXCOMMAND 8
char cmd[MAXCOMMAND];         // these are for handling and processing serial commands
char param[MAXCOMMAND];
char line[MAXCOMMAND];
int eoc = 0;    // end of command
int idx = 0;    // index into command string

// DB18B20 info
#define MaxProbes 2       // maximum probes per channel, really only supports 1
#define TEMP_PRECISION 10 // Set the DS18B20s precision to 0.25 of a degree 
DeviceAddress tpAddress;  // used to send precision setting to specific sensor
int tprobe1 = 0;          // indicate if there is a probe attached to that channel
float ch1tempval;         // temperature value for probe
float lasttempval = 20.0; // holds previous temperature value - used if ismoving and if temp request < 10s apart
bool displayincelsius = true;  // on lcd/tft, temperatures displayed in Celsius or Fahrenheit

// disable the stepper motor outputs - coil power off
void disableoutput() {
  digitalWrite(myEnable, HIGH);
  // control via ENABLE pin, but this turns off indexer inside DRV8825
  // which means the stepper motor will loose power and move position to the nearest full step
  // not an issue if using full steps but major issue if using microstepping as will cause change
  // in focus position
}

// enable the stepper motor outputs - coil power on
void enableoutput() {
  digitalWrite(myEnable, LOW);
}

// Move stepper anticlockwise
void anticlockwise() {
  if ( !myfocuser.ReverseDirection ) {
    analogWrite(gledOUT, 1023 ); // indicate a pulse by lighting the green LED
    digitalWrite(myDir, LOW );
    digitalWrite(myStep, 1 );
    delayMicroseconds(stepontime);
    digitalWrite(myStep, 0 );
    analogWrite(gledOUT, 0 ); // turn LED off
  }
  else {
    analogWrite(bledIN, 1023 );  // indicate a pulse by lighting the blue LED
    digitalWrite(myDir, HIGH );
    digitalWrite(myStep, 1 );
    delayMicroseconds(stepontime);
    digitalWrite(myStep, 0 );
    analogWrite(bledIN, 0 );  // turn LED off
  }
}

// Move stepper clockwise
void clockwise() {
  if ( !myfocuser.ReverseDirection ) {
    analogWrite(bledIN, 1023 );  // indicate a pulse by lighting the blue LED
    digitalWrite(myDir, HIGH );
    digitalWrite(myStep, 1 );
    delayMicroseconds(stepontime);
    digitalWrite(myStep, 0 );
    analogWrite(bledIN, 0 );  // turn LED off
  }
  else {
    analogWrite(gledOUT, 1023 ); // indicate a pulse by lighting the green LED
    digitalWrite(myDir, LOW );
    digitalWrite(myStep, 1 );
    delayMicroseconds(stepontime);
    digitalWrite(myStep, 0 );
    analogWrite(gledOUT, 0 ); // turn LED off
  }
}

// set the microstepping mode
void setstepmode() {
  switch ( myfocuser.stepmode ) {
    case 1:      // full step
      digitalWrite(myM0, 0);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 0);
      break;
    case 2:      // half step
      digitalWrite(myM0, 1);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 0);
      break;
    case 4:
      digitalWrite(myM0, 0);
      digitalWrite(myM1, 1);
      digitalWrite(myM2, 0);
      break;
    case 8:
      digitalWrite(myM0, 1);
      digitalWrite(myM1, 1);
      digitalWrite(myM2, 0);
      break;
    case 16:
      digitalWrite(myM0, 0);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 1);
      break;
    case 32:
      digitalWrite(myM0, 1);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 1);
      break;
    case 64:      // half step - not supported by this driver board
      digitalWrite(myM0, 1);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 0);
      break;
    case 128:      // half step - not supported by this driver board
      digitalWrite(myM0, 1);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 0);
      break;
    default:      // half step
      digitalWrite(myM0, 1);
      digitalWrite(myM1, 0);
      digitalWrite(myM2, 0);
      break;
  }
}

// ASCOM Serial Commands
void processCommand(String command) {
  memset(cmd, 0, MAXCOMMAND);
  memset(param, 0, MAXCOMMAND);
  int len = strlen(line);
  if (len >= 2) {
    strncpy(cmd, line, 2);
  }
  if (len > 2) {
    strncpy(param, line + 2, len - 2);
  }

  memset(line, 0, MAXCOMMAND);
  eoc = 0;
  idx = 0;

  // get the current focuser position
  if (!strcasecmp(cmd, "GP")) {
    char tempString[6];
    sprintf(tempString, "%04X", currentPosition);
    Serial.print(tempString);
    Serial.print("#");
  }

  // motor is moving - 1 if moving, 0 otherwise
  else if (!strcasecmp(cmd, "GI")) {
    if (isMoving ) {
      Serial.print("01#");
    }
    else {
      Serial.print("00#");
    }
  }

  // initiate a move to the target position
  else if (!strcasecmp(cmd, "FG")) {
    gotonewposition = true;
    isMoving = true;
  }

  // stop a move - HALT
  else if (!strcasecmp(cmd, "FQ")) {
    gotonewposition = false;
    isMoving = false;
    targetPosition = currentPosition;
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // home the motor, hard-coded, ignore parameters
  // not implemented in INDI driver
  else if (!strcasecmp(cmd, "PH")) {
    gotonewposition = true;
    isMoving = true;
    targetPosition = 0;
  }

  // get the new motor position (target)
  // not implemented in INDI driver
  else if (!strcasecmp(cmd, "GN")) {
    char tempString[6];
    sprintf(tempString, "%04X", targetPosition);
    Serial.print(tempString);
    Serial.print("#");
  }

  // set fullstep mode
  else if (!strcasecmp(cmd, "SF")) {
    myfocuser.stepmode = 1;
    setstepmode();
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // set halfstep mode
  else if (!strcasecmp(cmd, "SH")) {
    myfocuser.stepmode = 2;
    setstepmode();
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // whether half-step is enabled or not, always return "00"
  else if (!strcasecmp(cmd, "GH")) {
    if ( myfocuser.stepmode == 2 )
      Serial.print("FF#");
    else
      Serial.print("00#");
  }

  // get stepmode
  // myfocuser command
  else if (!strcasecmp(cmd, "GS")) {
    char tempString[6];
    sprintf(tempString, "%02X", myfocuser.stepmode);
    Serial.print(tempString);
    Serial.print("#");
  }

  // set stepmode
  // myfocuser command
  else if (!strcasecmp(cmd, "SS")) {
    pos = hexstr2long(param);
    myfocuser.stepmode = pos;
    setstepmode();
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // set current position to received position - no move SPXXXX
  // in INDI driver, only used to set to 0 SP0000 in reset()
  else if (!strcasecmp(cmd, "SP")) {
    pos = hexstr2long(param);
    if ( pos > maxSteps ) pos = maxSteps;
    if ( pos < 0 ) pos = 0;
    currentPosition = pos;
    targetPosition = pos;
    // signal that the focuser position has changed and should be saved to eeprom
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start 30s time interval
    gotonewposition = false;
    isMoving = false;
  }

  // set new target position SNXXXX - this is a move command
  // but must be followed by a FG command to start the move
  else if (!strcasecmp(cmd, "SN")) {
    pos = hexstr2long(param);
    if ( pos > maxSteps ) pos = maxSteps;
    if ( pos < 0 ) pos = 0;
    targetPosition = pos;
    gotonewposition = false;
    isMoving = false;
  }

  // get the current temperature - moonlite compatible
  else if (!strcasecmp(cmd, "GT")) {
    char tempString[6];
    long tempcurrentMillis = millis();
    // Send the command to get temperatures from DS18B20 probe
    if ( isMoving == false )   // if focuser is not moving
    {
      if ( tprobe1 == 1 ) {    // if there is a temperature probe
        // if this request is less than 5s from last one
        if ( (tempcurrentMillis - temppreviousMillis) > myfocuser.tempwaitdelay) {
          sensor1.requestTemperatures();
          // get channel 1 temperature, always in celsius
          ch1tempval = sensor1.getTempCByIndex(0);
          // sometimes sensor returns -127, not sure why, so check and use last reading if this is the case
          int tempval;
          tempval = (int) ch1tempval;
          if ( tempval < -40 )
            ch1tempval = lasttempval;

          lasttempval = ch1tempval;         // remember last reading
        }
        else  // request is less than tempwaitdelay so return previous value
        {
          ch1tempval = lasttempval;      // default temp if there is no probe
        }
      }
      else  // return default temp if there is no probe
        ch1tempval = lasttempval;
    }
    else // default temp if its moving - last known temp val
      ch1tempval = lasttempval;

    ch1tempval = (ch1tempval * 2) + tempoffsetval;
    sprintf(tempString, "%04X", (int) ch1tempval);
    Serial.print(tempString);
    Serial.print("#");
  }

  // get the current temperature
  // myfocuser command
  else if (!strcasecmp(cmd, "GZ")) {
    char tempString[6];
    long tempcurrentMillis = millis();
    // Send the command to get temperatures from DS18B20 probe
    if ( isMoving == false )   // if focuser is not moving
    {
      if ( tprobe1 == 1 ) {    // if there is a temperature probe
        // if this request is less than 5s from last one
        if ( (tempcurrentMillis - temppreviousMillis) > myfocuser.tempwaitdelay) {
          sensor1.requestTemperatures();
          // get channel 1 temperature, always in celsius
          ch1tempval = sensor1.getTempCByIndex(0);
          // sometimes sensor returns -127, not sure why, so check and use last reading if this is the case
          int tempval;
          tempval = (int) ch1tempval;
          if ( tempval < -40 )
            ch1tempval = lasttempval;

          lasttempval = ch1tempval;         // remember last reading
        }
        else  // request is less than tempwaitdelay so return previous value
        {
          ch1tempval = lasttempval;      // default temp if there is no probe
        }
      }
      else  // return default temp if there is no probe
        ch1tempval = lasttempval;
    }
    else // default temp if its moving - last known temp val
      ch1tempval = lasttempval;

    Serial.print(ch1tempval);
    Serial.print("#");
  }

  // firmware value Moonlite
  else if (!strcasecmp(cmd, "GV")) {
    Serial.print("10#");
  }

  // firmware value
  // myfocuser command
  else if (!strcasecmp(cmd, "GF")) {
    Serial.println(programName);
    Serial.print(programVersion + "#");
  }

  // get the current motor step delay, only values of 02, 04, 08, 10, 20
  // not used so just return 02
  else if (!strcasecmp(cmd, "GD")) {
    char tempString[6];
    sprintf(tempString, "%02X", stepdelay);
    Serial.print(tempString);
    Serial.print("#");
  }

  // set step delay, only acceptable values are 02, 04, 08, 10, 20 which
  // correspond to a stepping delay of 250, 125, 63, 32 and 16 steps
  // per second respectively. Moonlite only
  else if (!strcasecmp(cmd, "SD")) {
    pos = hexstr2long(param);
    stepdelay = (int) pos;
  }

  // set temperature co-efficient XX
  else if (!strcasecmp(cmd, "SC")) {
    // do nothing, ignore
    pos = hexstr2long(param);
    tempcomp = (int) pos;
  }

  // get temperature co-efficient XX
  else if (!strcasecmp(cmd, "GC")) {
    // do nothing, ignore
    char tempString[6];
    sprintf(tempString, "%02X", tempcomp);
    Serial.print(tempString);
    Serial.print("#");
  }

  // + activate temperature compensation focusing
  else if (!strcasecmp(cmd, "+")) {
    // ignore
  }

  // - disable temperature compensation focusing
  else if (!strcasecmp(cmd, "-")) {
    // ignore
  }

  // temperature calibration offset POXX in 0.5 degree increments (hex)
  else if (!strcasecmp(cmd, "PO")) {
    // Moonlite only
    // this adds/subtracts an offset from the temperature reading in 1/2 degree C steps
    // FA -3, FB -2.5, FC -2, FD -1.5, FE -1, FF -.5, 00 0, 01 0.5, 02 1.0, 03 1.5, 04 2.0, 05 2.5, 06 3.0
    tempoffsetval = 0.0;
    if ( param == "FA" ) tempoffsetval = -3.0;
    else if ( param == "FB") tempoffsetval = -2.5;
    else if ( param == "FC") tempoffsetval = -2.0;
    else if ( param == "FD") tempoffsetval = -1.5;
    else if ( param == "FE") tempoffsetval = -1.0;
    else if ( param == "FF") tempoffsetval = -0.5;
    else if ( param == "00") tempoffsetval = 0.0;
    else if ( param == "01") tempoffsetval = 0.5;
    else if ( param == "02") tempoffsetval = 1.0;
    else if ( param == "03") tempoffsetval = 1.5;
    else if ( param == "04") tempoffsetval = 2.0;
    else if ( param == "05") tempoffsetval = 2.5;
    else if ( param == "06") tempoffsetval = 3.0;
  }

  // set new maxSteps position SMXXXX
  // myfocuser command
  else if (!strcasecmp(cmd, "SM")) {
    pos = hexstr2long(param);
    if ( pos > maxFocuserLimit ) pos = maxFocuserLimit;
    // avoid setting maxSteps too low
    if ( pos < 1000 ) pos = 1000;
    // for NEMA17 at 400 steps this would be 5 full rotations of focuser knob
    // for 28BYG-28 this would be less than 1/2 a revolution of focuser knob
    maxSteps = pos;
    // check maxIncement in case its larger
    if ( maxIncrement > maxSteps ) maxIncrement = maxSteps;
    // signal that the focuser position has changed and should be saved to eeprom
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start 30s time interval
  }

  // get the MaxSteps
  // myfocuser command
  else if (!strcasecmp(cmd, "GM")) {
    char tempString[6];
    sprintf(tempString, "%04X", maxSteps);
    Serial.print(tempString);
    Serial.print("#");
  }

  // set new maxIncrement SYXXXX
  // myfocuser command
  else if (!strcasecmp(cmd, "SY")) {
    pos = hexstr2long(param);
    // ignore
    maxIncrement = pos;
  }

  // get the maxIncrement - set to MaxSteps
  // myfocuser command
  else if (!strcasecmp(cmd, "GY")) {
    char tempString[6];
    sprintf(tempString, "%04X", maxIncrement);
    Serial.print(tempString);
    Serial.print("#");
  }

  // get the coilPwr setting
  // myfocuser command
  else if (!strcasecmp(cmd, "GO")) {
    String tempString;
    if ( myfocuser.coilPwr )
      tempString = "01#";
    else
      tempString = "00#";
    Serial.print(tempString);
  }

  // set the coilPwr setting
  // myfocuser command
  else if (!strcasecmp(cmd, "SO")) {
    pos = hexstr2long(param);
    if ( pos == 0 )
      myfocuser.coilPwr = false;
    else
      myfocuser.coilPwr = true;
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // get the Reverse Direction setting
  // myfocuser command
  else if (!strcasecmp(cmd, "GR")) {
    String tempString;
    if ( myfocuser.ReverseDirection )
      tempString = "01#";
    else
      tempString = "00#";
    Serial.print(tempString);
  }

  // set the Reverse Direction setting
  // myfocuser command
  else if (!strcasecmp(cmd, "SR")) {
    pos = hexstr2long(param);
    if ( pos == 0 )
      myfocuser.ReverseDirection = false;
    else
      myfocuser.ReverseDirection = true;
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // set motorSpeed - time delay between pulses, acceptable values are 00, 01 and 02 which
  // correspond to a slow, med, high
  // myfocuser command
  else if (!strcasecmp(cmd, "MS")) {
    pos = hexstr2long(param);
    if ( pos == 0 )
      motorSpeed = motorSpeedSlow;
    else if ( pos == 1 )
      motorSpeed = motorSpeedMed;
    else if (pos == 2 )
      motorSpeed = motorSpeedFast;
    else
      motorSpeed = motorSpeedMed;
  }
  // get the tempwaitdelay setting
  // not moonlite command
  else if (!strcasecmp(cmd, "GE")) {
    char tempString[12];
    sprintf(tempString, "%04X", myfocuser.tempwaitdelay);
    Serial.print(tempString);
    Serial.print("#");
  }

  // set the tempwaitdelay setting
  // not moonlite command
  else if (!strcasecmp(cmd, "SE")) {
    pos = hexstr2long(param);
    // bounds check to 2000-10000 2s-10s
    if ( pos < 2000L )
      pos = 20000L;
    if ( pos > 10000L )
      pos = 10000L;
    myfocuser.tempwaitdelay = pos;
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();   // start time interval
  }

  // LED backlight value, always return "00" - moonlite
  // not implemented in INDI driver
  else if (!strcasecmp(cmd, "GB")) {
    Serial.print("00#");
  }

  else if ( !strcasecmp( cmd, "DM")) {
    pos = hexstr2long(param);
    if ( pos > 0 )
      displayincelsius = false;
    else
      displayincelsius = true;
  }

  // troubleshooting only - print currentaddr value, use in serial monitor mode is best
  else if (!strcasecmp(cmd, "XY")) {
    Serial.println();
    Serial.print("Current EEPROM Addr=");
    Serial.println(currentaddr);
    Serial.print("nlocations=");
    Serial.println(nlocations);
    Serial.print("Number of writes=");
    Serial.println(numwrites);
    Serial.print("focuser=");
    Serial.println(myfocuser.fposition);
    Serial.print("maxSteps=");
    Serial.println(myfocuser.maxstep);
    Serial.print("maxIncrement=");
    Serial.println(maxIncrement);
    Serial.print("Stepmode=");
    Serial.println(myfocuser.stepmode);
    Serial.print("CoilPower=");
    Serial.println(myfocuser.coilPwr);
    Serial.print("ReverseDirection=");
    Serial.println(myfocuser.ReverseDirection);
    Serial.print("Motorspeed=");
    Serial.println(motorSpeed);
    Serial.print("Tempprobe=");
    Serial.println(tprobe1);
    Serial.print("Display Mode=");
    if ( displayincelsius == true ) Serial.println("Celsius"); else Serial.println("Fahrenheit");
    Serial.print("Tempwaitdelay=");
    Serial.println(myfocuser.tempwaitdelay);
    Serial.print("#");
  }

  // troubleshooting only - reset focuser defaults
  // myfocuser command
  else if (!strcasecmp(cmd, "XZ")) {
    currentaddr = 0;
    myfocuser.validdata = 99;
    myfocuser.maxstep = 10000L;
    myfocuser.fposition = 5000L;
    myfocuser.coilPwr = true;
    myfocuser.ReverseDirection = false;
    myfocuser.stepmode = 1;
    myfocuser.tempwaitdelay = 5000L;
    // now write the data to EEPROM
    EEPROM_writeAnything(currentaddr, myfocuser);    // update values in EEPROM
    numwrites++;
    // Set focuser defaults.
    currentPosition = myfocuser.fposition;
    targetPosition = myfocuser.fposition;
    maxSteps = myfocuser.maxstep;
    displayincelsius = true;
  }
}

// convert hex string to long int
long hexstr2long(char *line) {
  long ret = 0;

  ret = strtol(line, NULL, 16);
  return (ret);
}
// Setup
void setup() {
  // initialize serial for ASCOM
  Serial.begin(9600);

  // this is mandatory as ASCOM driver from Moonlite needs a quick response on connect
  // response for :GB#, to try and let ASCOM driver startup properly
  Serial.print("00#");

  // turn ON the Buzzer - provide power ON beep
  analogWrite( Buzzer, 700 );
  // turn ON both LEDS as power on cycle indicator
  analogWrite( bledIN, 1023 );
  analogWrite( gledOUT, 1023 );

  eoc = 0;
  idx = 0;
  isMoving = false;
  memset(line, 0, MAXCOMMAND);

  pinMode(  myDir, OUTPUT );
  pinMode(  myStep, OUTPUT );
  pinMode(  myM0, OUTPUT );
  pinMode(  myM1, OUTPUT );
  pinMode(  myM2, OUTPUT );

  // set direction and step to low
  digitalWrite( myDir, 0 );
  digitalWrite( myStep, 0 );

  boardstate = OUTPUTENABLED;
  pinMode( myEnable, OUTPUT );    // enable the driver board
  enableoutput();

  currentaddr = 0;    // start at 0 if not found later
  found = false;
  writenow = false;
  datasize = sizeof( myfocuser );    // should be 14 bytes
  nlocations = EEPROMSIZE / datasize;  // for AT328P = 1024 / datasize = 73 locations

  numwrites = 0;

  for (int lp1 = 0; lp1 < nlocations; lp1++ ) {
    int addr = lp1 * datasize;
    EEPROM_readAnything( addr, myfocuser );
    // check to see if the data is valid
    if ( myfocuser.validdata == 99 ) {
      // data was erased so write some default values
      currentaddr = addr;
      found = true;
    }
  }
  if ( found == true ) {
    // set the focuser back to the previous settings
    // done after this in one hit
    // mark current eeprom address as invalid and use next one
    // each time focuser starts it will read current storage, set it to invalid, goto next location and
    // write values to there and set it to valid - so it doesnt always try to use same locations over and
    // over and destroy the eeprom
    // using it like an array of [0-nlocations], ie 100 storage locations for 1k EEPROM
    EEPROM_readAnything( currentaddr, myfocuser );
    myfocuser.validdata = 0;
    EEPROM_writeAnything(currentaddr, myfocuser);    // update values in EEPROM
    numwrites++;
    // goto next free address and write data
    currentaddr += datasize;
    // bound check the eeprom storage and if greater than last index [0-EEPROMSIZE-1] then set to 0
    if ( currentaddr >= (nlocations * datasize) ) currentaddr = 0;

    myfocuser.validdata = 99;
    EEPROM_writeAnything(currentaddr, myfocuser);    // update values in EEPROM
    numwrites++;
  }
  else {
    // set defaults because not found
    myfocuser.validdata = 99;
    myfocuser.maxstep = 10000L;
    myfocuser.fposition = 5000L;
    myfocuser.coilPwr = true;
    myfocuser.ReverseDirection = false;
    myfocuser.stepmode = 1;    // full stepping
    myfocuser.tempwaitdelay = 5000L;
    // now write the data to EEPROM
    EEPROM_writeAnything(currentaddr, myfocuser);    // update values in EEPROM
    numwrites++;
  }

  // Set focuser defaults from saved values in EEPROM.
  currentPosition = myfocuser.fposition;
  targetPosition = myfocuser.fposition;
  maxSteps = myfocuser.maxstep;

  // these will be updated by ASCOM driver or Applciation when connect()
  // myfocuser.coilPwr
  // myfocuser.ReverseDirection
  // myfocuser.stepmode
  setstepmode();

  tempoffsetval = 0;

  // start temperature sensors DS18B20
  tprobe1 = 0;        // set probe indicator NOT FOUND
  sensor1.begin();    // start the temperature sensor1
  sensor1.getDeviceCount();    // should return 1 if probe connected
  for (int i = 0; i < MaxProbes; i++) {  // look for probes
    // Search the wire for address
    if (sensor1.getAddress(tpAddress, i)) { // found?
      tprobe1 = 1;  // there is a probe1
      sensor1.setResolution(tpAddress, TEMP_PRECISION);
      // set the resolution for that probe to 10bit 0.25degC
    }
  }

  temppreviousMillis = millis();  // keep track of when last temp conversion was done

  // do a temperature conversion now so can be ready to return valid value if requested
  // Send the command to get temperatures from DS18B20 probes
  if ( tprobe1 == 1 )    // if there is a temperature probe
  {
    sensor1.requestTemperatures();
    // get channel 1 temperature
    ch1tempval = sensor1.getTempCByIndex(0);
    lasttempval = ch1tempval;         // remember last reading
  }
  else  // return default temp if there is no probe
  {
    ch1tempval = lasttempval;
  }

  // set lcd/tft to display temperature readings in celsius as default
  displayincelsius = true;

  // setup defaults for rotary encoder
  myrotaryencoder.minimum(-1000);
  myrotaryencoder.maximum(1000);
  encswval = 1;
  setLED(1);
  pinMode( encoderSWPin, INPUT_PULLUP );

  // setup the IN/OUT LEDS
  analogWrite( bledIN, 0 );
  analogWrite( gledOUT, 0 );
  analogWrite( Buzzer, 0);
}

// Main Loop
void loop() {
  // check rotary encoder
  // do not update if focuser is already moving - just wait
  int lp;
  if ( isMoving == false )
  {
    lp = myrotaryencoder.getposition();
    if ( lp != 0 ) {
      targetPosition = targetPosition + (myrotaryencoder.getposition() * encswval);
      if ( targetPosition < 1 )
        targetPosition = 0;
      if ( targetPosition >= maxSteps )
        targetPosition = maxSteps;
      myrotaryencoder.setposition(0);    // reset to avoid counting more than once
      gotonewposition = true;           // needed to GO once focus position has changed
    }
  }

  if (!digitalRead(encoderSWPin)) {
    delay(50);
    if (!digitalRead(encoderSWPin)) {
      long encswstart = millis();    // start time of encoder sw press
      while ( !digitalRead(encoderSWPin)) {
        encswpress = millis();      // end time of encoder sw press
        long encswtime = encswpress - encswstart;  // how long enc sw was pressed
        if (encswtime > 1500) {
          motorSpeed = motorSpeedMed;
          currentPosition = 0;
          targetPosition = 0;
          isMoving = false;
          digitalWrite(led1, HIGH);
          digitalWrite(led2, HIGH);
          digitalWrite(led3, HIGH);
          analogWrite( Buzzer, 1023 );
          delay(200);
          analogWrite( Buzzer,  0);
          digitalWrite(led1, LOW);
          digitalWrite(led2, LOW);
          digitalWrite(led3, LOW);
          encswval = 100;
        }
      }
      if ( encswval == 100 ) {
        setLED(1);
        motorSpeed = motorSpeedSlow;
        encswval = 1;
      }
      else if (encswval == 1 ) {
        motorSpeed = motorSpeedMed;
        encswval = 10;
        setLED(2);
      }
      else if (encswval == 10 ) {
        motorSpeed = motorSpeedFast;
        encswval = 100;
        setLED(3);
      }
    }
  }

  // process the command string when a hash arrives:
  if (eoc) {
    processCommand(line);
    memset(line, 0, MAXCOMMAND);
    eoc = false;
  }

  // only move if new target has been set and FG
  // remember that SN can set new target but focuser should not move till it gets FG
  if ((targetPosition != currentPosition) && (gotonewposition == true) ) {
    if ( boardstate == OUTPUTDISABLED )       // if board is not enabled, we need to enable it else
    { // it will not step
      boardstate = OUTPUTENABLED;
      enableoutput();              // have to enable driver board
    }
    isMoving = true;              // focuser is moving
    writenow = true;             // updating of EEPROM ON
    previousMillis = millis();    // keep updating previousMillis whilst focuser is moving

    delay( motorSpeed );          // delay between pulses to stepper thus controlling speed

    // Going Anticlockwise to lower position
    if (targetPosition < currentPosition) {
      anticlockwise();
      currentPosition--;
    }

    // Going Clockwise to higher position
    if (targetPosition > currentPosition) {
      // do not need to check if > maximumPosition as its done when a target command is receieved
      clockwise();
      currentPosition++;
    }
  } else {
    // focuser is NOT moving now, move is completed
    isMoving = false;
    gotonewposition = false;
    if ( writenow == true ) {    // time to update eeprom

      // decide if we have waited 30s after the last move, if so, update the EEPROM
      long currentMillis = millis();
      if ( (currentMillis - previousMillis > interval) && (writenow == true) ) {
        previousMillis = currentMillis;    // update the timestamp

        // copy current settings and write the data to EEPROM
        myfocuser.validdata = 99;
        myfocuser.fposition = currentPosition;
        myfocuser.maxstep = maxSteps;
        EEPROM_writeAnything(currentaddr, myfocuser);    // update values in EEPROM
        numwrites++;
        writenow = false;
      }
    }
    if ( myfocuser.coilPwr == false ) {
      boardstate = OUTPUTDISABLED;
      disableoutput();      // release the stepper coils to save power
    }
  }
}

// SerialEvent occurs whenever new data comes in the serial RX.
void serialEvent() {
  // : starts the command, # ends the command, do not store these in the command buffer
  // read the command until the terminating # character
  while (Serial.available() && !eoc) {
    inChar = Serial.read();
    if (inChar != '#' && inChar != ':') {
      line[idx++] = inChar;
      if (idx >= MAXCOMMAND) {
        idx = MAXCOMMAND - 1;
      }
    }
    else {
      if (inChar == '#') {
        eoc = 1;
        idx = 0;
      }
    }
  }
}

void setLED(int i) {
  digitalWrite(led1, LOW);
  digitalWrite(led2, LOW);
  digitalWrite(led3, LOW);
  switch (i) {
    case 1:
      digitalWrite(led1, HIGH);
      break;
    case 2:
      digitalWrite(led2, HIGH);
      break;
    case 3:
      digitalWrite(led3, HIGH);
      break;
  }
}
